@@ -40,6 +40,7 @@ my $authParameters;
 my $issuerParameters;
 my $samlServiceParameters;
 my $oidcServiceParameters;
+my $casServiceParameters = [];
 my $defaultValues;
 
 my $attributes = Lemonldap::NG::Manager::Build::Attributes::attributes();
@@ -86,6 +87,9 @@ sub run {
   switch(tpl){
 ';
 
+    my $reverseScanResult =
+      $self->reverseScan( Lemonldap::NG::Manager::Build::Tree::tree(), '', {} );
+
     # To build confTree.js, each special node is scanned from
     # Lemonldap::NG::Manager::Build::CTrees
     foreach my $node ( sort keys %$mainTree ) {
@@ -101,7 +105,7 @@ sub run {
     return $tmp;
 ";
 
-        # Second step, Manager/Constants.pm file will contain datas issued from
+        # Second step, Manager/Constants.pm file will contain data issued from
         # this scan
         my $ra = Regexp::Assemble->new;
 
@@ -158,7 +162,7 @@ sub run {
     my $exportedVars = '$'
       . join( 'Keys $',
         'simpleHash', 'doubleHash', 'specialNode', sort keys %cnodesRe )
-      . 'Keys $specialNodeHash $authParameters $issuerParameters $samlServiceParameters $oidcServiceParameters';
+      . 'Keys $specialNodeHash $authParameters $issuerParameters $samlServiceParameters $oidcServiceParameters $casServiceParameters';
     print F <<EOF;
 # This file is generated by $module. Don't modify it by hand
 package Lemonldap::NG::Common::Conf::ReConstants;
@@ -179,6 +183,8 @@ our \$specialNodeHash = {
     samlSPMetaDataNodes  => [qw(samlSPMetaDataXML samlSPMetaDataExportedAttributes samlSPMetaDataOptions)],
     oidcOPMetaDataNodes  => [qw(oidcOPMetaDataJSON oidcOPMetaDataJWKS oidcOPMetaDataOptions oidcOPMetaDataExportedVars)],
     oidcRPMetaDataNodes  => [qw(oidcRPMetaDataOptions oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims)],
+    casSrvMetaDataNodes  => [qw(casSrvMetaDataOptions casSrvMetaDataExportedVars)],
+    casAppMetaDataNodes  => [qw(casAppMetaDataOptions casAppMetaDataExportedVars)],
 };
 
 EOF
@@ -223,6 +229,14 @@ EOF
 
     printf STDERR $format, $self->defaultValuesFile;
     $defaultValues->{locationRules} = $attributes->{locationRules}->{default};
+    foreach ( keys %$attributes ) {
+        if (    not /(?:MetaData|vhost)/
+            and $attributes->{$_}->{default}
+            and not $reverseScanResult->{$_} )
+        {
+            $defaultValues->{$_} = $attributes->{$_}->{default};
+        }
+    }
     my $defaultAttr = mydump( $defaultValues, 'defaultValues' );
     $defaultAttr = "# This file is generated by $module. Don't modify it by hand
 package Lemonldap::NG::Common::Conf::DefaultValues;
@@ -252,17 +266,21 @@ $defaultAttr}
 
     printf STDERR $format, $self->confConstantsFile;
     $ra = Regexp::Assemble->new;
-    foreach ( @simpleHashKeys, sort keys %cnodesRe ) {
+    foreach ( @simpleHashKeys, @doubleHashKeys, sort keys %cnodesRe ) {
         $ra->add($_);
     }
     foreach (
-        qw(exportedHeaders locationRules post vhostOptions
+        qw(
+        exportedHeaders locationRules post vhostOptions
         samlIDPMetaDataXML samlIDPMetaDataExportedAttributes
         samlIDPMetaDataOptions samlSPMetaDataXML
         samlSPMetaDataExportedAttributes samlSPMetaDataOptions
         oidcOPMetaDataJSON oidcOPMetaDataJWKS oidcOPMetaDataOptions
         oidcOPMetaDataExportedVars oidcRPMetaDataOptions
-        oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims)
+        oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims
+        casAppMetaDataExportedVars casAppMetaDataOptions
+        casSrvMetaDataExportedVars casSrvMetaDataOptions
+        )
       )
     {
         $ra->add($_);
@@ -376,12 +394,7 @@ $managerAttr}
     printf STDERR $format, $self->reverseTreeFile;
     open( F, ">", $self->reverseTreeFile ) or die($!);
     $jsonEnc->pretty(0);
-    print F $jsonEnc->encode(
-        $self->reverseScan(
-            Lemonldap::NG::Manager::Build::Tree::tree(),
-            '', {}
-        )
-    );
+    print F $jsonEnc->encode($reverseScanResult);
     close F;
     print STDERR "done\n";
 }
@@ -405,6 +418,7 @@ sub mydump {
     my ( $obj, $subname ) = @_;
     my $t = Dumper($obj);
     $t =~ s/^\s*(?:use strict;|package .*?;|)\n//gm;
+    $t =~ s/\n\s*BEGIN.*?\}\n/\n/sg;
     $t =~ s/^\$VAR1\s*=/sub $subname {\n    return/;
     return $t;
 }
@@ -438,7 +452,7 @@ sub scanTree {
         elsif ( ref($leaf) ) {
             $jleaf->{title} = $jleaf->{id} = $leaf->{title};
             $jleaf->{type} = $leaf->{form} if ( $leaf->{form} );
-            if ( $leaf->{title} =~ /^((?:oidc|saml)Service)MetaData$/ ) {
+            if ( $leaf->{title} =~ /^((?:oidc|saml|cas)Service)MetaData$/ ) {
                 no strict 'refs';
                 my @tmp = $self->scanLeaf( $leaf->{nodes} );
                 ${ $1 . 'Parameters' } = \@tmp;
@@ -495,6 +509,7 @@ sub scanTree {
             push @sessionTypes, $1
               if ( $leaf =~ /^(.*)(?<!notification)StorageOptions$/ );
             my $attr = $attributes->{$leaf} or die("Missing attribute $leaf");
+
             #print STDERR "| $attr->{documentation}  |  $leaf  |\n";
             $jleaf = { id => "$prefix$leaf", title => $leaf };
             unless ( $attr->{type} ) {
@@ -628,8 +643,8 @@ system.
 
 =head1 DESCRIPTION
 
-Lemonldap::NG::Manager::Build is used only to build javascript files and
-Lemonldap::NG constants Perl files. It has to be launched after each change.
+Lemonldap::NG::Manager::Build is only used to build javascript files and
+Lemonldap::NG constants Perl files. It must be launched after each change.
 
 =head2 DEVELOPER CORNER
 
@@ -642,7 +657,7 @@ To add a new parameter, you have to:
 =item declare its position in the tree in Manager/Build/Tree.pm (or
 Manager/Build/CTrees.pm for complex nodes);
 
-=item refresh files using this (or launch any build makefile target at the
+=item refresh files by using this (or launch any build makefile target at the
 root of the Lemonldap::NG project sources).
 
 =back
@@ -651,18 +666,18 @@ See below for details.
 
 =head3 Files generated
 
-The `scripts/jsongenerator.pl` file uses Lemonldap::NG::Manager::Build::Attributes,
+`scripts/jsongenerator.pl` file uses Lemonldap::NG::Manager::Build::Attributes,
 Lemonldap::NG::Manager::Build::Tree and Lemonldap::NG::Manager::Build::CTrees to generate
 
 =over
 
 =item `site/htdocs/static/struct.json`:
 
-the main file that contains the tree view;
+Main file containing the tree view;
 
 =item `site/htdocs/static/js/conftree.js`:
 
-generates sub tree for virtualhosts and SAML and OpenID-Connect partners;
+generates Virtualhosts, SAML and OpenID-Connect partners sub-trees;
 
 =item `site/htdocs/static/reverseTree.json`:
 
@@ -688,30 +703,30 @@ first configuration in file format;
 
 =head3 Attribute declaration
 
-Set your new attribute as a key of attributes() function that points to a hash
+set your new attribute as a key of attributes() function that points to a hash
 ref containing:
 
 =over
 
 =item type (required):
 
-the type of the content. It must be declared in sub
-types() in the same file (except if attribute embeds its own tests) and must
-correspond to a form stored in the static/forms/ directory;
+content type must be declared in sub types() in the same file
+(except if attribute embeds its own tests) and must match
+to a form stored in static/forms/ directory;
 
 =item help (optional):
 
-the relative HTML path to the help page (relative to
+Relative HTML path to help page (relative to
 /doc/pages/documentation/<version>/);
 
 =item default (recommended):
 
-a default value to set if not defined;
+default value to set if not defined;
 
 =item select (optional):
 
 required only if type is `select`. In this case, it
-must contains an array of { k => <keyName>, v => <display name> } hashref
+must contain an array of { k => <keyName>, v => <display name> } hashref
 
 =item documentation (recommended):
 
@@ -720,10 +735,10 @@ some words for other developpers
 =item test (optional):
 
 if test is not defined for this type or if test must
-be more restrictive, set her a regular expression or a subroutine. Arguments
-passed to subroutine are (keyValue, newConf, currentKey), it returns 2
-arguments: a boolean result and a message (if non empty will be displayed as
-warning or error depending of result);
+be more restrictive, set here a regular expression or a subroutine. Arguments
+passed to subroutine are (keyValue, newConf, currentKey). It returns 2
+arguments: a boolean result and a message (if non empty message will be 
+displayed as warning or error depending of result);
 
 =item msgFail (optional):
 
@@ -732,30 +747,28 @@ error. Words to translate have to be written as so: __toTranslate__;
 
 =item keyTest (optional):
 
-for keys/values attributes, test to be applied on
-key;
+for keys/values attributes, test to be applied on key;
 
 =item keyMsgFail (optional):
 
-for regexp based key tests, same as msgFail for
-keys test;
+for regexp based key tests, same as msgFail for keys test;
 
 =back
 
-If you decide to declare a new type, you have to declare the following
+If you want to declare a new type, you have to declare following
 properties:
 
 =over
 
 =item test, msgFail, keyTest, keyMsgFail as shown above,
 
-=item form: the form to use if it doesn't have the same name.
+=item form: form to use if it doesn't have the same name.
 
 =back
 
-=head3 Tree positioning
+=head3 Tree location
 
-The tree is now very simple: it contains nodes and leaf. Leaf are designed only
+The tree is now very simple: it contains nodes and leaves. Leaves are designed only
 by their attribute name. All description must be done in the file described
 above. Nodes are array member designed as this:
 
@@ -791,13 +804,13 @@ array of sub nodes and leaf attached to this node
 
 =item group:
 
-must never be used in conjunction with nodes. Array of leafs only
+must never be used in conjunction with nodes. Array of leaves only
 to be displayed in the same form
 
 =item nodes_cond:
 
-array of sub nodes that will be displayed with a filter. Not
-yet documented here, see the source code of site/htdocs/static/js/filterFunctions.js.
+array of sub nodes that will be displayed with a filter. Not yet
+documented here, see the source code of site/htdocs/static/js/filterFunctions.js.
 
 =item nodes_filter:
 
@@ -820,7 +833,7 @@ L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 Note that if you want to post a ticket for a conf upload problem, please
 see L<Lemonldap::NG::Manager::Conf::Parser> before.
