@@ -10,11 +10,11 @@
 #
 # All other ideas have to be set in Manager/Build/Attributes.pm !
 
-# DON'T FORGET TO RUN jsongenerator.pl AFTER EACH CHANGE
+# DON'T FORGET TO RUN "make json" AFTER EACH CHANGE
 
 package Lemonldap::NG::Manager::Build::CTrees;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 sub cTrees {
     return {
@@ -25,10 +25,11 @@ sub cTrees {
             {
                 title => 'vhostOptions',
                 help  => 'configvhost.html#options',
+                form  => 'simpleInputContainer',
                 nodes => [
                     'vhostPort',        'vhostHttps',
                     'vhostMaintenance', 'vhostAliases',
-                    'vhostType',
+                    'vhostType',        'vhostAuthnLevel',
                 ],
             },
         ],
@@ -56,7 +57,8 @@ sub cTrees {
                 nodes => [
                     "samlIDPMetaDataOptionsAdaptSessionUtime",
                     "samlIDPMetaDataOptionsForceUTF8",
-                    "samlIDPMetaDataOptionsStoreSAMLToken"
+                    "samlIDPMetaDataOptionsStoreSAMLToken",
+                    "samlIDPMetaDataOptionsUserAttribute"
                 ]
             },
             {
@@ -121,7 +123,8 @@ sub cTrees {
                         form  => 'simpleInputContainer',
                         nodes => [
                             "samlSPMetaDataOptionsEncryptionMode",
-                            "samlSPMetaDataOptionsEnableIDPInitiatedURL"
+                            "samlSPMetaDataOptionsEnableIDPInitiatedURL",
+                            "samlSPMetaDataOptionsRule",
                         ]
                     }
                 ]
@@ -203,15 +206,43 @@ sub cTrees {
                         title => 'logout',
                         form  => 'simpleInputContainer',
                         nodes => [
+                            'oidcRPMetaDataOptionsPostLogoutRedirectUris',
                             'oidcRPMetaDataOptionsLogoutUrl',
                             'oidcRPMetaDataOptionsLogoutType',
                             'oidcRPMetaDataOptionsLogoutSessionRequired',
                         ]
                     },
+                    'oidcRPMetaDataOptionsRule',
                 ]
             },
             'oidcRPMetaDataOptionsExtraClaims',
         ],
+        casSrvMetaDataNode => [
+            {
+                title => 'casSrvMetaDataOptions',
+                form  => 'simpleInputContainer',
+                nodes => [
+                    'casSrvMetaDataOptionsUrl',
+                    'casSrvMetaDataOptionsRenew',
+                    'casSrvMetaDataOptionsGateway',
+                    'casSrvMetaDataOptionsDisplayName',
+                    'casSrvMetaDataOptionsIcon',
+                ]
+            },
+            'casSrvMetaDataExportedVars',
+            'casSrvMetaDataOptionsProxiedServices',
+        ],
+        casAppMetaDataNode => [
+            {
+                title => 'casAppMetaDataOptions',
+                form  => 'simpleInputContainer',
+                nodes => [
+                    'casAppMetaDataOptionsService',
+                    'casAppMetaDataOptionsRule'
+                ]
+            },
+            'casAppMetaDataExportedVars',
+        ],
     };
 }
 
