@@ -5,7 +5,8 @@ use Mouse;
 use Data::Dumper;
 use Lemonldap::NG::Common::Conf::ReConstants;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
+$Data::Dumper::Useperl = 1;
 
 extends('Lemonldap::NG::Manager::Cli::Lib');
 
@@ -55,6 +56,7 @@ sub set {
         if ( ref $oldValue ) {
             die "$key seems to be a hash, modification refused";
         }
+        $oldValue //= '';
         push @list, [ $key, $oldValue, $pairs{$key} ];
     }
     unless ( $self->yes ) {
@@ -103,6 +105,14 @@ sub addKey {
         elsif ( $#path == 1 ) {
             $new->{ $path[0] }->{ $path[1] }->{ $el->[1] } = $el->[2];
         }
+        elsif ( $#path == 2 ) {
+            $new->{ $path[0] }->{ $path[1] }->{ $path[2] }->{ $el->[1] } =
+              $el->[2];
+        }
+        elsif ( $#path == 3 ) {
+            $new->{ $path[0] }->{ $path[1] }->{ $path[2] }->{ $path[3] }
+              ->{ $el->[1] } = $el->[2];
+        }
         else {
             die $el->[0] . " has too many levels. Aborting";
         }
@@ -130,10 +140,44 @@ sub delKey {
     foreach my $el (@list) {
         my @path = split $sep, $el->[0];
         if ( $#path == 0 ) {
-            delete $new->{ $path[0] }->{ $el->[1] };
+            if (   exists $new->{ $path[0] }
+                && exists $new->{ $path[0] }->{ $el->[1] } )
+            {
+                delete $new->{ $path[0] }->{ $el->[1] }
+                  if exists $new->{ $path[0] }->{ $el->[1] };
+            }
         }
         elsif ( $#path == 1 ) {
-            delete $new->{ $path[0] }->{ $path[1] }->{ $el->[1] };
+            if (   exists $new->{ $path[0] }
+                && exists $new->{ $path[0] }->{ $path[1] }
+                && exists $new->{ $path[0] }->{ $path[1] }->{ $el->[1] } )
+            {
+                delete $new->{ $path[0] }->{ $path[1] }->{ $el->[1] };
+            }
+        }
+        elsif ( $#path == 2 ) {
+            if (   exists $new->{ $path[0] }
+                && exists $new->{ $path[0] }->{ $path[1] }
+                && exists $new->{ $path[0] }->{ $path[1] }->{ $path[2] }
+                && exists $new->{ $path[0] }->{ $path[1] }->{ $path[2] }
+                ->{ $el->[1] } )
+            {
+                delete $new->{ $path[0] }->{ $path[1] }->{ $path[2] }
+                  ->{ $el->[1] };
+            }
+        }
+        elsif ( $#path == 3 ) {
+            if (   exists $new->{ $path[0] }
+                && exists $new->{ $path[0] }->{ $path[1] }
+                && exists $new->{ $path[0] }->{ $path[1] }->{ $path[2] }
+                && exists $new->{ $path[0] }->{ $path[1] }->{ $path[2] }
+                ->{ $path[3] }
+                && exists $new->{ $path[0] }->{ $path[1] }->{ $path[2] }
+                ->{ $path[3] }->{ $el->[1] } )
+            {
+                delete $new->{ $path[0] }->{ $path[1] }->{ $path[2] }
+                  ->{ $path[3] }->{ $el->[1] };
+            }
         }
         else {
             die $el->[0] . " has too many levels. Aborting";
@@ -199,7 +243,12 @@ sub _save {
     unless ( $parser->testNewConf() ) {
         printf STDERR "Modifications rejected: %s:\n", $parser->{message};
     }
-    my $s = $self->mgr->confAcc->saveConf( $new, { force => $self->force } );
+    my $saveParams = { force => $self->force };
+    if ( $self->force and $self->cfgNum ) {
+        $saveParams->{cfgNum}      = $self->cfgNum;
+        $saveParams->{cfgNumFixed} = 1;
+    }
+    my $s = $self->mgr->confAcc->saveConf( $new, %$saveParams );
     if ( $s > 0 ) {
         print STDERR "Saved under number $s\n";
         $parser->{status} = [ $self->mgr->applyConf($new) ];
@@ -220,8 +269,6 @@ sub _save {
 sub run {
     my $self = shift;
 
-    print STDERR "VERY EXPERIMENTAL FEATURE, prefer web interface\n";
-
     # Options simply call corresponding accessor
     my $args = {};
     while ( $_[0] =~ s/^--?// ) {
@@ -272,8 +319,8 @@ __END__
 
 =encoding utf8
 
-Lemonldap::NG::Manager::Cli - EXPERIMENTAL command line manager for
-Lemonldap::NG web SSO system.
+Lemonldap::NG::Manager::Cli - Command line manager for Lemonldap::NG web SSO
+system.
 
 =head1 SYNOPSIS
 
@@ -297,8 +344,8 @@ or use llng-manager-cli provides with this package.
 Lemonldap::NG::Manager provides a web interface to manage Lemonldap::NG Web-SSO
 system.
 
-Lemonldap::NG Manager::Cli provides an EXPERIMENTAL command line client to read
-or modify configuration.
+Lemonldap::NG Manager::Cli provides a command line client to read or modify
+configuration.
 
 =head1 METHODS
 
@@ -370,7 +417,7 @@ Other links: L<Lemonldap::NG::Manager>, L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
