@@ -19,7 +19,7 @@ use feature 'state';
 
 extends 'Lemonldap::NG::Common::Conf::RESTServer';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 #############################
 # I. INITIALIZATION METHODS #
@@ -43,6 +43,7 @@ sub addRoutes {
             ':cfgNum' => [
                 qw(virtualHosts samlIDPMetaDataNodes samlSPMetaDataNodes
                   applicationList oidcOPMetaDataNodes oidcRPMetaDataNodes
+                  casSrvMetaDataNodes casAppMetaDataNodes
                   authChoiceModules grantSessionRules combModules
                   openIdIDPList)
             ]
@@ -73,7 +74,7 @@ sub addRoutes {
 
 ##@method public PSGI-JSON-response newRSAKey($req)
 # Return a hashref containing private and public keys
-# The posted datas must contain a JSON object containing
+# The posted data must contain a JSON object containing
 # {"password":"newpassword"}
 #
 #@param $req Lemonldap::NG::Common::PSGI::Request object
@@ -331,7 +332,7 @@ sub applyConf {
     }
     return {} unless (%reloadUrls);
 
-    $self->ua->timeout(3);
+    $self->ua->timeout( $newConf->{reloadTimeout} );
 
     # Parse apply values
     while ( my ( $host, $request ) = each %reloadUrls ) {
@@ -375,6 +376,7 @@ sub diff {
     my @cfgNum =
       ( scalar( $req->param('conf1') ), scalar( $req->param('conf2') ) );
     my @conf;
+    $self->logger->debug(" Loading confs");
 
     # Load the 2 configurations
     for ( my $i = 0 ; $i < 2 ; $i++ ) {
@@ -394,8 +396,14 @@ sub diff {
         }
     }
     require Lemonldap::NG::Manager::Conf::Diff;
-    return $self->sendJSONresponse( $req,
-        [ Lemonldap::NG::Manager::Conf::Diff->diff( $conf[0], $conf[1] ) ] );
+    return $self->sendJSONresponse(
+        $req,
+        [
+            $self->Lemonldap::NG::Manager::Conf::Diff::diff(
+                $conf[0], $conf[1]
+            )
+        ]
+    );
 }
 
 1;
