@@ -1,6 +1,6 @@
 package Lemonldap::NG::Manager::Conf::Zero;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 sub zeroConf {
     my ( $domain, $sessionDir, $persistentSessionDir, $notificationDir ) = @_;
@@ -108,7 +108,8 @@ sub zeroConf {
         },
         'macros' => {
             '_whatToTrace' =>
-'$_auth eq \'SAML\' ? "$_user\\@$_idpConfKey" : $_auth eq \'OpenIDConnect\' ? "$_user\\@$_oidcConnectedRP" : "$_user"'
+'$_auth eq \'SAML\' ? "$_user\\@$_idpConfKey" : $_auth eq \'OpenIDConnect\' ? "$_user\\@$_oidcConnectedRP" : "$_user"',
+            'UA' => '$ENV{HTTP_USER_AGENT}'
         },
         'notificationStorageOptions' => {
             'dirName' => $notificationDir
@@ -128,7 +129,7 @@ sub zeroConf {
             'LockDirectory' => "$persistentSessionDir/lock"
         },
         'reloadUrls' => {
-            "reload.$domain" => "http://reload.$domain/reload"
+            "localhost" => "http://reload.$domain/reload"
         },
         'sessionDataToRemember' => {},
         'notification'          => 1,
@@ -155,7 +156,7 @@ sub zeroConf {
                 '^/logout' => 'logout_sso'
             },
             "manager.$domain" => {
-                'default'                                  => '$uid eq "dwho"',
+                'default' => '$uid eq "dwho" or $uid eq "rtyler"',
                 '(?#Configuration)^/(manager\.html|conf/)' => '$uid eq "dwho"',
                 '(?#Sessions)/sessions' => '$uid eq "dwho" or $uid eq "rtyler"',
                 '(?#Notifications)/notifications' =>
@@ -167,10 +168,8 @@ sub zeroConf {
         'cookieName'    => 'lemonldap',
         'cfgAuthor'     => 'The LemonLDAP::NG team',
         'cfgVersion'    => $VERSION,
-        'exportedVars'  => {
-            'UA' => 'HTTP_USER_AGENT'
-        },
-        'portalSkin' => 'bootstrap',
+        'exportedVars'  => {},
+        'portalSkin'    => 'bootstrap',
         'portalSkinBackground' =>
           '1280px-Cedar_Breaks_National_Monument_partially.jpg',
         'mailUrl'                    => "http://auth.$domain/resetpwd",
