@@ -10,13 +10,14 @@ use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::Session;
 use Lemonldap::NG::Common::PSGI::Constants;
 use Lemonldap::NG::Common::Conf::ReConstants;
+use Lemonldap::NG::Common::IPv6;
 
 use feature 'state';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib',
   'Lemonldap::NG::Common::Session::REST';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 #############################
 # I. INITIALIZATION METHODS #
@@ -37,6 +38,15 @@ sub addRoutes {
       ->addRoute(
         sessions => { ':sessionType' => { ':sessionId' => 'delSession' } },
         ['DELETE']
+      )
+
+      # DELETE OIDC CONSENT
+      ->addRoute(
+        sessions => {
+            OIDCConsent =>
+              { ':sessionType' => { ':sessionId' => 'delOIDCConsent' } }
+        },
+        ['DELETE']
       );
 
     $self->setTypes($conf);
@@ -47,9 +57,34 @@ sub addRoutes {
 }
 
 #######################
-# II. DISPLAY METHODS #
+# II. CONSENT METHODS #
 #######################
 
+sub delOIDCConsent {
+
+    my ( $self, $req, $session, $skey ) = @_;
+
+    my $mod = $self->getMod($req)
+      or return $self->sendError( $req, undef, 400 );
+
+    my $params = $req->parameters();
+    my $epoch  = $params->{epoch};
+    my $rp     = $params->{rp};
+
+    if ( $rp =~ /\b[\w-]+\b/ and defined $epoch ) {
+        $self->logger->debug(
+            "Call procedure deleteOIDCConsent with RP=$rp and epoch=$epoch");
+        return $self->deleteOIDCConsent( $req, $session, $skey );
+    }
+    else {
+        return $self->sendError( $req, undef, 400 );
+    }
+}
+
+########################
+# III. DISPLAY METHODS #
+########################
+
 sub sessions {
     my ( $self, $req, $session, $skey ) = @_;
 
@@ -74,12 +109,12 @@ sub sessions {
     my @fields = ( '_httpSessionType', $self->{ipField}, $whatToTrace );
     if ( my $groupBy = $params->{groupBy} ) {
         $groupBy =~ s/^substr\((\w+)(?:,\d+(?:,\d+)?)?\)$/$1/
-          or $groupBy =~ s/^net4\((\w+),\d\)$/$1/;
+          or $groupBy =~ s/^net(?:4|6|)\(([\w:]+),\d+(?:,\d+)?\)$/$1/;
         $groupBy =~ s/^_whatToTrace$/$whatToTrace/o
           or push @fields, $groupBy;
     }
     elsif ( my $order = $params->{orderBy} ) {
-        $order =~ s/^net4\((\w+)\)$/$1/;
+        $order =~ s/^net(?:4|6|)\(([\w:]+)\)$/$1/;
         $order =~ s/^_whatToTrace$/$whatToTrace/o
           or push @fields, split( /, /, $order );
     }
@@ -99,19 +134,31 @@ sub sessions {
           : ( $s => $params->{$_} );
     } keys %$params;
     $filters{_session_kind} = $type;
-
-    # Check if a '*' is required
-    my $function = 'searchOn';
-    $function = 'searchOnExpr' if ( grep /\*/, values %filters );
+    push @fields, keys(%filters);
+    {
+        my %seen;
+        @fields = grep { !$seen{$_}++ } @fields;
+    }
 
     # For now, only one argument can be passed to
     # Lemonldap::NG::Common::Apache::Session so just the first filter is
     # used
     my ($firstFilter) = sort {
-            $a eq '_session_kind' ? 1
-          : $b eq '_session_kind' ? -1
+            $filters{$a} =~ m#^[\w:]+/\d+\*?$# ? 1
+          : $filters{$b} =~ m#^[\w:]+/\d+\*?$# ? -1
+          : $a eq '_session_kind'              ? 1
+          : $b eq '_session_kind'              ? -1
           : $a cmp $b
     } keys %filters;
+
+    # Check if a '*' is required
+    my $function = 'searchOn';
+    $function = 'searchOnExpr'
+      if ( grep { /\*/ and not m#^[\w:]+/\d+\*?$# }
+        ( $filters{$firstFilter} ) );
+    $self->logger->debug(
+        "First filter: $firstFilter = $filters{$firstFilter} ($function)");
+
     $res =
       Lemonldap::NG::Common::Apache::Session->$function( $moduleOptions,
         $firstFilter, $filters{$firstFilter}, @fields );
@@ -126,13 +173,26 @@ sub sessions {
         }
     ) unless ( $res and %$res );
 
-    delete $filters{$firstFilter};
+    delete $filters{$firstFilter}
+      unless ( grep { /\*/ and not m#^[\w:]+/\d+\*?$# }
+        ( $filters{$firstFilter} ) );
     foreach my $k ( keys %filters ) {
-        $filters{$k} =~ s/\*/\.\*/g;
-        foreach my $session ( keys %$res ) {
-            if ( $res->{$session}->{$k} ) {
+        $self->logger->debug("Removing unless $k =~ /^$filters{$k}\$/");
+        if ( $filters{$k} =~ m#^([\w:]+)/(\d+)\*?$# ) {
+            my ( $net, $bits ) = ( $1, $2 );
+            foreach my $session ( keys %$res ) {
                 delete $res->{$session}
-                  unless ( $res->{$session}->{$k} =~ /^$filters{$k}$/ );
+                  unless ( net6( $res->{$session}->{$k}, $bits ) eq $net );
+            }
+        }
+        else {
+            $filters{$k} =~ s/\./\\./g;
+            $filters{$k} =~ s/\*/\.\*/g;
+            foreach my $session ( keys %$res ) {
+                if ( $res->{$session}->{$k} ) {
+                    delete $res->{$session}
+                      unless ( $res->{$session}->{$k} =~ /^$filters{$k}$/ );
+                }
             }
         }
     }
@@ -208,7 +268,7 @@ sub sessions {
             $group = $field;
         }
 
-        # Subnets
+        # Subnets IPv4
         elsif ( $group =~ /^net4\((\w+),(\d)\)$/ ) {
             my $field = $1;
             my $nb    = $2 - 1;
@@ -221,6 +281,33 @@ sub sessions {
             $group = $field;
         }
 
+        # Subnets IPv6
+        elsif ( $group =~ /^net6\(([\w:]+),(\d)\)$/ ) {
+            my $field = $1;
+            my $bits  = $2;
+            foreach my $k ( keys %$res ) {
+                $r->{ net6( $res->{$k}->{$field}, $bits ) . "/$bits" }++
+                  if ( isIPv6( $res->{$k}->{$field} ) );
+            }
+        }
+
+        # Both IPv4 and IPv6
+        elsif ( $group =~ /^net\(([\w:]+),(\d+),(\d+)\)$/ ) {
+            my $field = $1;
+            my $bits  = $2;
+            my $nb    = $3 - 1;
+            foreach my $k ( keys %$res ) {
+                if ( isIPv6( $res->{$k}->{$field} ) ) {
+                    $r->{ net6( $res->{$k}->{$field}, $bits ) . "/$bits" }++;
+                }
+                elsif ( $res->{$k}->{$field} =~ /^((((\d+)\.\d+)\.\d+)\.\d+)$/ )
+                {
+                    my @d = ( $4, $3, $2, $1 );
+                    $r->{ $d[$nb] }++;
+                }
+            }
+        }
+
         # Simple field groupBy query
         elsif ( $group =~ /^\w+$/ ) {
             eval {
@@ -240,8 +327,17 @@ qq{Use of an uninitialized attribute "$group" to group sessions},
 
         # Build result
         $res = [
-            sort { $a->{value} cmp $b->{value} }
-            map { { value => $_, count => $r->{$_} } } keys %$r
+            sort {
+                my @a = ( $a->{value} =~ /^(\d+)(?:\.(\d+))*$/ );
+                my @b = ( $b->{value} =~ /^(\d+)(?:\.(\d+))*$/ );
+                ( @a and @b )
+                  ? ( $a[0] <=> $b[0]
+                      or $a[1] <=> $b[1]
+                      or $a[2] <=> $b[2]
+                      or $a[3] <=> $b[3] )
+                  : $a->{value} cmp $b->{value}
+              }
+              map { { value => $_, count => $r->{$_} } } keys %$r
         ];
     }
 
@@ -253,24 +349,30 @@ qq{Use of an uninitialized attribute "$group" to group sessions},
             my $tmp = { session => $_ };
             foreach my $f (@fields) {
                 my $s = $f;
-                $s =~ s/^net4\((\w+)\)$/$1/;
+                $s =~ s/^net(?:4|6|)\(([\w:]+)\)$/$1/;
                 $tmp->{$s} = $res->{$_}->{$s};
             }
             $tmp
         } keys %$res;
         while ( my $f = pop @fields ) {
             if ( $f =~ s/^net4\((\w+)\)$/$1/ ) {
+                @r = sort { cmpIPv4( $a->{$f}, $b->{$f} ); } @r;
+            }
+            elsif ( $f =~ s/^net6\(([:\w]+)\)$/$1/ ) {
+                @r = sort { expand6( $a->{$f} ) cmp expand6( $b->{$f} ); } @r;
+            }
+            elsif ( $f =~ s/^net\(([:\w]+)\)$/$1/ ) {
                 @r = sort {
-                    my @a = split /\./, $a->{$f};
-                    my @b = split /\./, $b->{$f};
-                    my $cmp = 0;
-                  F: for ( my $i = 0 ; $i < 4 ; $i++ ) {
-                        if ( $a[$i] != $b[$i] ) {
-                            $cmp = $a[$i] <=> $b[$i];
-                            last F;
-                        }
-                    }
-                    $cmp;
+                    my $ip1 = $a->{$f};
+                    my $ip2 = $b->{$f};
+                    isIPv6($ip1)
+                      ? (
+                          isIPv6($ip2)
+                        ? expand6($ip1) cmp expand6($ip2)
+                        : -1
+                      )
+                      : isIPv6($ip2) ? 1
+                      :                cmpIPv4( $ip1, $ip2 );
                 } @r;
             }
             else {
@@ -301,4 +403,17 @@ qq{Use of an uninitialized attribute "$group" to group sessions},
     );
 }
 
+sub cmpIPv4 {
+    my @a = split /\./, $_[0];
+    my @b = split /\./, $_[1];
+    my $cmp = 0;
+  F: for ( my $i = 0 ; $i < 4 ; $i++ ) {
+        if ( $a[$i] != $b[$i] ) {
+            $cmp = $a[$i] <=> $b[$i];
+            last F;
+        }
+    }
+    $cmp;
+}
+
 1;
